#ifndef _GAME_H_
#define _GAME_H_

#include "tetrominoes.h"

#define BOARD_ROWS	20
#define BOARD_COLS	10

#define EEPROMADDR	0

enum game_state { GAME_START_SCREEN, GAME_RUNNING, GAME_PAUSED, GAME_END_SCREEN };

/**
 * Function to initialise the internal functionality of the game.
 */
void game_init(void);

/**
 * Function to update the game logic each frame.
 */
void game_loop(void);

/**
 * Signal asynchronously that the down button was pressed.
 */
void game_down_button_pressed(void);

/**
 * Signal asynchronously that the down button is held.
 */
void game_down_button_hold(void);

/**
 * Signal asynchronously that the left button was pressed.
 */
void game_left_button_pressed(void);

/**
 * Signal asynchronously that the left button is held.
 */
void game_left_button_hold(void);

/**
 * Signal asynchronously that the right button was pressed.
 */
void game_right_button_pressed(void);

/**
 * Signal asynchronously that the right button is held.
 */
void game_right_button_hold(void);

/**
 * Signal asynchronously that the clockwise button was pressed.
 */
void game_cw_button_pressed(void);

/**
 * Signal asynchronously that the clockwise button is held.
 */
void game_cw_button_hold(void);

/**
 * Signal asynchronously that the counterclockwise button was pressed.
 */
void game_ccw_button_pressed(void);

/**
 * Signal asynchronously that the counterclockwise button is held.
 */
void game_ccw_button_hold(void);

/**
 * Signal asynchronously that the pause button was pressed.
 */
void game_up_button_pressed(void);

/**
 * Signal asynchronously that the pause button is held.
 */
void game_up_button_hold(void);

/**
 * Get the pointer to the game board.
 */
uint8_t *game_get_board(void);

/**
 * Get the pointer to the current falling tetromino.
 */
struct tetro *game_get_tetro(void);

/**
 * Get the current level of the game.
 */
uint8_t game_get_level(void);

/**
 * Get the current score of the game.
 */
uint32_t game_get_score(void);

/**
 * Get the highest score achieved in a game.
 */
uint32_t game_get_high_score(void);

/**
 * Get the state of the game.
 */
enum game_state game_get_state(void);

#endif /* _GAME_H_ */
