#include <avr/pgmspace.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "renderer.h"
#include "nokia5110.h"
#include "tetrominoes.h"
#include "utils.h"

void renderer_init(void)
{
	nokia_lcd_init();
}

/**
 * Get the coordinates where to draw minoes from the board.
 * @param bx x coordinate in matrix
 * @param by y coordinate in matrix
 * @param sx x coordinate on screen returned as parameter
 * @param sy y coordinate on screen returned as parameter
 */
static void translate_coords(uint8_t bx, uint8_t by, uint8_t *sx, uint8_t *sy)
{
	*sx = 80 - 4 * bx;
	*sy = 40 - 4 * by;
}

static void render_outline(void)
{
	nokia_lcd_draw_rectagle(0, 2,  84, 4);
	nokia_lcd_draw_rectagle(0, 44, 84, 46);
}

static void render_tetro(void)
{
	int i, j;
	uint8_t x, y;
	struct tetro *tetro = game_get_tetro();

	for (i = 0; i < tetro->size; ++i) {
		// Skip lines outside the screen
		if (tetro->row + i < 0 || tetro->row + i >= BOARD_ROWS + 1) {
			continue;
		}

		for (j = 0; j < tetro->size; ++j) {
			// Skip rows outside the screen
			if (tetro->col + j < 0 || tetro->col + j >= BOARD_COLS) {
				continue;
			}

			if (tetro_get_data(tetro, i, j, tetro->rot) == 1) {
				translate_coords(tetro->row + i, tetro->col + j, &x, &y);
				nokia_lcd_draw_square_pretty(x, y);
			}
		}
	}
}

static void render_board(void)
{
	int i, j;
	uint8_t x, y;
	char str[15];
	uint8_t *board = game_get_board();
	uint8_t level = game_get_level();
	uint32_t score = game_get_score();

	render_outline();
	render_tetro();
	
	// Draw score and level at the top of the screen
	memset(str, 0, sizeof(str));
	nokia_lcd_set_cursor(0, 46);
	sprintf(str, "%lu", score);
	nokia_lcd_write_string(str, 1);

	memset(str, 0, sizeof(str));
	nokia_lcd_set_cursor(8, 46);
	sprintf(str, "%u", level);
	nokia_lcd_write_string(str, 1);

	// Draw all minoes as pretty squares
	for (i = 0; i < BOARD_ROWS; ++i) {
		for (j = 0; j < BOARD_COLS; ++j) {
			if (board[i * BOARD_COLS + j] == 1) {
				translate_coords(i, j, &x, &y);
				nokia_lcd_draw_square_pretty(x, y);
			}
		}
	}
}

void render_scene(void)
{
	enum game_state state = game_get_state();
	char str[15];
	uint32_t score, high_score;

	nokia_lcd_clear();
	switch (state) {
	case GAME_START_SCREEN:
		nokia_lcd_set_cursor(0, 46);
		nokia_lcd_write_string("TETRIS", 1);
		nokia_lcd_set_cursor(35, 46);
		nokia_lcd_write_string("Press", 1);
		nokia_lcd_set_cursor(44, 46);
		nokia_lcd_write_string("any", 1);
		nokia_lcd_set_cursor(53, 46);
		nokia_lcd_write_string("button", 1);
		nokia_lcd_set_cursor(62, 46);
		nokia_lcd_write_string("to", 1);
		nokia_lcd_set_cursor(71, 46);
		nokia_lcd_write_string("start", 1);
		break;

	case GAME_RUNNING:
		render_board();
		break;

	case GAME_PAUSED:
		nokia_lcd_set_cursor(40, 46);
		nokia_lcd_write_string("PAUSED", 1);
		break;

	case GAME_END_SCREEN:
		nokia_lcd_set_cursor(0, 46);
		nokia_lcd_write_string("GAME", 1);
		nokia_lcd_set_cursor(8, 46);
		nokia_lcd_write_string("OVER", 1);

		nokia_lcd_set_cursor(30, 46);
		nokia_lcd_write_string("Score", 1);
		nokia_lcd_set_cursor(38, 46);
		score = game_get_score();
		memset(str, 0, sizeof(str));
		sprintf(str, "%ld", score);
		nokia_lcd_write_string(str, 1);

		nokia_lcd_set_cursor(60, 46);
		nokia_lcd_write_string("High", 1);
		nokia_lcd_set_cursor(68, 46);
		nokia_lcd_write_string("score", 1);
		nokia_lcd_set_cursor(76, 46);
		high_score = game_get_high_score();
		memset(str, 0, sizeof(str));
		sprintf(str, "%ld", high_score);
		nokia_lcd_write_string(str, 1);
		break;
	}
	nokia_lcd_render();
}
