#ifndef __TETROMINO_H__
#define __TETROMINO_H__

#include <avr/pgmspace.h>

#define TETRO_NUM	7

enum tetro_type {T_I, T_J, T_L, T_O, T_S, T_T, T_Z, T_NONE};

struct tetro {
	// name of tetromino shape
	enum tetro_type type;
	// vectorised 3D matrix having [rotation][row coordinate][col coordinate]
	const uint8_t *data;
	// maximum between piece width and length
	uint8_t size;

	// current index of rotation
	uint8_t rot;
	// maximun number of rotation
	uint8_t max_rot;

	// row coordinate in or outside board matrix
	int8_t row;
	// col coordinate in or outside board matrix
	int8_t col;
};

extern const struct tetro tetro_i;
extern const struct tetro tetro_j;
extern const struct tetro tetro_l;
extern const struct tetro tetro_o;
extern const struct tetro tetro_s;
extern const struct tetro tetro_t;
extern const struct tetro tetro_z;
extern const struct tetro tetro_none;

/**
 * Get data of a mino from the tetromino.
 * @param tetro pointer to tetromino
 * @param row row coordinate of mino
 * @param col column coordinate of mino
 * @param rot rotation of the tetromino
 * @return 
 */
uint8_t tetro_get_data(struct tetro *tetro, int row, int col, uint8_t rot);

#endif /* __TETROMINO_H__ */
